@@ -372,7 +372,6 @@ xmlNanoHTTPScanProxy(const char *URL) {
 static xmlNanoHTTPCtxtPtr
 xmlNanoHTTPNewCtxt(const char *URL) {
     xmlNanoHTTPCtxtPtr ret;
-    xmlChar *escaped;
 
     ret = (xmlNanoHTTPCtxtPtr) xmlMalloc(sizeof(xmlNanoHTTPCtxt));
     if (ret == NULL) return(NULL);
@@ -383,13 +382,7 @@ xmlNanoHTTPNewCtxt(const char *URL) {
     ret->fd = -1;
     ret->ContentLength = -1;
 
-    escaped = xmlURIEscapeStr(BAD_CAST URL, BAD_CAST"@/:=?;#%&");
-    if (escaped != NULL) {
-	xmlNanoHTTPScanURL(ret, (const char *) escaped);
-	xmlFree(escaped);
-    } else {
-	xmlNanoHTTPScanURL(ret, URL);
-    }
+    xmlNanoHTTPScanURL(ret, URL);
 
     return(ret);
 }
@@ -491,7 +484,7 @@ xmlNanoHTTPRecv(xmlNanoHTTPCtxtPtr ctxt) {
 
     while (ctxt->state & XML_NANO_HTTP_READ) {
 	if (ctxt->in == NULL) {
-	    ctxt->in = (char *) xmlMalloc(65000 * sizeof(char));
+	    ctxt->in = (char *) xmlMallocAtomic(65000 * sizeof(char));
 	    if (ctxt->in == NULL) {
 	        ctxt->last = -1;
 		xmlGenericError( xmlGenericErrorContext, 
@@ -1132,7 +1125,7 @@ retry:
     if (contentType && *contentType)
 	blen += strlen(*contentType) + 16;
     blen += strlen(method) + strlen(ctxt->path) + 24;
-    bp = xmlMalloc(blen);
+    bp = xmlMallocAtomic(blen);
     if ( bp == NULL ) {
         xmlNanoHTTPFreeCtxt( ctxt );
 	xmlGenericError( xmlGenericErrorContext,
