@@ -11,6 +11,8 @@
 #include <string.h>
 #include <stdarg.h>
 
+#include <assert.h>
+
 #if defined (_WIN32) && !defined(__CYGWIN__)
 #ifdef _MSC_VER
 #include <winsock2.h>
@@ -81,12 +83,20 @@
 #include <libxml/DOCBparser.h>
 #endif
 #include <libxml/globals.h>
+#include <libxml/xmlreader.h>
+#ifdef LIBXML_SCHEMAS_ENABLED
+#include <libxml/relaxng.h>
+#endif
+
+#ifndef XML_XML_DEFAULT_CATALOG
+#define XML_XML_DEFAULT_CATALOG "file:///etc/xml/catalog"
+#endif
 
 #ifdef LIBXML_DEBUG_ENABLED
-static int debug = 0;
 static int shell = 0;
 static int debugent = 0;
 #endif
+static int debug = 0;
 static int copy = 0;
 static int recovery = 0;
 static int noent = 0;
@@ -95,6 +105,10 @@ static int nowrap = 0;
 static int valid = 0;
 static int postvalid = 0;
 static char * dtdvalid = NULL;
+#ifdef LIBXML_SCHEMAS_ENABLED
+static char * relaxng = NULL;
+static xmlRelaxNGPtr relaxngschemas = NULL;
+#endif
 static int repeat = 0;
 static int insert = 0;
 static int compress = 0;
@@ -124,6 +138,8 @@ static int dropdtd = 0;
 static int catalogs = 0;
 static int nocatalogs = 0;
 #endif
+static int stream = 0;
+static int chkregister = 0;
 static const char *output = NULL;
 
 
@@ -137,7 +153,7 @@ static const char *output = NULL;
 #ifdef HAVE_SYS_TIME_H
 #ifdef HAVE_FTIME
 
-int
+static int
 my_gettimeofday(struct timeval *tvp, void *tzp)
 {
 	struct timeb timebuffer;
@@ -505,6 +521,7 @@ xmlHTMLValidityWarning(void *ctx, const char *msg, ...)
  * 			Shell Interface					*
  * 									*
  ************************************************************************/
+#ifdef LIBXML_DEBUG_ENABLED
 /**
  * xmlShellReadline:
  * @prompt:  the prompt value
@@ -545,6 +562,7 @@ xmlShellReadline(char *prompt) {
     return(ret);
 #endif
 }
+#endif /* LIBXML_DEBUG_ENABLED */
 
 /************************************************************************
  * 									*
@@ -563,7 +581,117 @@ static void myClose(FILE *f) {
 
 /************************************************************************
  * 									*
- * 			Test processing					*
+ * 			Stream Test processing				*
+ * 									*
+ ************************************************************************/
+static int count = 0;
+static int elem, attrs;
+
+static void processNode(xmlTextReaderPtr reader) {
+    xmlChar *name, *value;
+
+    name = xmlTextReaderName(reader);
+    if (name == NULL)
+	name = xmlStrdup(BAD_CAST "--");
+
+    value = xmlTextReaderValue(reader);
+
+    printf("%d %d %s %d %d", 
+	    xmlTextReaderDepth(reader),
+	    xmlTextReaderNodeType(reader),
+	    name,
+	    xmlTextReaderIsEmptyElement(reader),
+	    xmlTextReaderHasValue(reader));
+    xmlFree(name);
+    if (value == NULL)
+	printf("\n");
+    else {
+	printf(" %s\n", value);
+	xmlFree(value);
+    }
+}
+
+static void streamFile(char *filename) {
+    xmlTextReaderPtr reader;
+    int ret;
+
+    if (count) {
+	elem = 0;
+	attrs = 0;
+    }
+
+    reader = xmlNewTextReaderFilename(filename);
+    if (reader != NULL) {
+	if (valid)
+	    xmlTextReaderSetParserProp(reader, XML_PARSER_VALIDATE, 1);
+	else
+	    xmlTextReaderSetParserProp(reader, XML_PARSER_LOADDTD, 1);
+	if (relaxng != NULL) {
+	    if (timing) {
+		startTimer();
+	    }
+	    ret = xmlTextReaderRelaxNGValidate(reader, relaxng);
+	    if (ret < 0) {
+		xmlGenericError(xmlGenericErrorContext,
+			"Relax-NG schema %s failed to compile\n", relaxng);
+		relaxng = NULL;
+	    }
+	    if (timing) {
+		endTimer("Compiling the schemas");
+	    }
+	}
+
+	/*
+	 * Process all nodes in sequence
+	 */
+	if (timing) {
+	    startTimer();
+	}
+	ret = xmlTextReaderRead(reader);
+	while (ret == 1) {
+	    if (debug)
+		processNode(reader);
+	    ret = xmlTextReaderRead(reader);
+	}
+	if (timing) {
+	    if ((valid) || (relaxng != NULL))
+		endTimer("Parsing and validating");
+	    else
+		endTimer("Parsing");
+	}
+
+	if (valid) {
+	    if (xmlTextReaderIsValid(reader) != 1) {
+		xmlGenericError(xmlGenericErrorContext,
+			"Document %s does not validate\n", filename);
+		progresult = 3;
+	    }
+	}
+	if (relaxng != NULL) {
+	    if (xmlTextReaderIsValid(reader) != 1) {
+		printf("%s fails to validate\n", filename);
+		progresult = 3;
+	    } else {
+		printf("%s validates\n", filename);
+	    }
+	}
+	/*
+	 * Done, cleanup and status
+	 */
+	xmlFreeTextReader(reader);
+	if (ret != 0) {
+	    printf("%s : failed to parse\n", filename);
+	    progresult = 1;
+	}
+    } else {
+	fprintf(stderr, "Unable to open %s\n", filename);
+	progresult = 1;
+    }
+}
+
+/************************************************************************
+ * 									*
+ * 			Tree Test processing				*
  * 									*
  ************************************************************************/
 static void parseAndPrintFile(char *filename) {
@@ -635,12 +763,11 @@ static void parseAndPrintFile(char *filename) {
 	    }
 	    if (f != NULL) {
 		int ret;
-	        int res, size = 3;
+	        int res, size = 1024;
 	        char chars[1024];
                 xmlParserCtxtPtr ctxt;
 
-		if (repeat)
-		    size = 1024;
+		/* if (repeat) size = 1024; */
 		res = fread(chars, 1, 4, f);
 		if (res > 0) {
 		    ctxt = xmlCreatePushParserCtxt(NULL, NULL,
@@ -743,8 +870,29 @@ static void parseAndPrintFile(char *filename) {
 	    doc = xmlParseMemory((char *) base, info.st_size);
 	    munmap((char *) base, info.st_size);
 #endif
-	} else
+	} else if (valid) {
+	    int ret;
+	    xmlParserCtxtPtr ctxt;
+
+	    ctxt = xmlCreateFileParserCtxt(filename);
+
+	    if (ctxt == NULL) {	      
+	      doc = NULL;
+	    } else {
+	      xmlParseDocument(ctxt);
+	      if (ctxt->valid == 0)
+		  progresult = 4;
+	      ret = ctxt->wellFormed;
+	      doc = ctxt->myDoc;
+	      xmlFreeParserCtxt(ctxt);
+	      if (!ret) {
+		xmlFreeDoc(doc);
+		doc = NULL;
+	      }
+	    }
+	} else {
 	    doc = xmlParseFile(filename);
+	}
     }
 
     /*
@@ -819,7 +967,7 @@ static void parseAndPrintFile(char *filename) {
 		    printf("%d element types can be inserted under root:\n",
 		           nb);
 		    for (i = 0;i < nb;i++) {
-			 printf("%s\n", list[i]);
+			 printf("%s\n", (char *) list[i]);
 		    }
 		}
 	    }
@@ -956,6 +1104,34 @@ static void parseAndPrintFile(char *filename) {
 	if ((timing) && (!repeat)) {
 	    endTimer("Validating");
 	}
+#ifdef LIBXML_SCHEMAS_ENABLED
+    } else if (relaxngschemas != NULL) {
+	xmlRelaxNGValidCtxtPtr ctxt;
+	int ret;
+
+	if ((timing) && (!repeat)) {
+	    startTimer();
+	}
+
+	ctxt = xmlRelaxNGNewValidCtxt(relaxngschemas);
+	xmlRelaxNGSetValidErrors(ctxt,
+		(xmlRelaxNGValidityErrorFunc) fprintf,
+		(xmlRelaxNGValidityWarningFunc) fprintf,
+		stderr);
+	ret = xmlRelaxNGValidateDoc(ctxt, doc);
+	if (ret == 0) {
+	    printf("%s validates\n", filename);
+	} else if (ret > 0) {
+	    printf("%s fails to validate\n", filename);
+	} else {
+	    printf("%s validation generated an internal error\n",
+		   filename);
+	}
+	xmlRelaxNGFreeValidCtxt(ctxt);
+	if ((timing) && (!repeat)) {
+	    endTimer("Validating");
+	}
+#endif
     }
 
 #ifdef LIBXML_DEBUG_ENABLED
@@ -1040,6 +1216,8 @@ static void usage(const char *name) {
     printf("\t--debug : dump a debug tree of the in-memory document\n");
     printf("\t--shell : run a navigating shell\n");
     printf("\t--debugent : debug the entities defined in the document\n");
+#else
+    printf("\t--debug : dump the nodes content when using --stream\n");
 #endif
     printf("\t--copy : used to test the internal copy implementation\n");
     printf("\t--recover : output what was parsable on broken XML documents\n");
@@ -1075,7 +1253,7 @@ static void usage(const char *name) {
 #ifdef LIBXML_CATALOG_ENABLED
     printf("\t--catalogs : use SGML catalogs from $SGML_CATALOG_FILES\n");
     printf("\t             otherwise XML Catalogs starting from \n");
-    printf("\t         file:///etc/xml/catalog are activated by default\n");
+    printf("\t         " XML_XML_DEFAULT_CATALOG " are activated by default\n");
     printf("\t--nocatalogs: deactivate all catalogs\n");
 #endif
     printf("\t--auto : generate a small doc on the fly\n");
@@ -1085,15 +1263,35 @@ static void usage(const char *name) {
     printf("\t--loaddtd : fetch external DTD\n");
     printf("\t--dtdattr : loaddtd + populate the tree with inherited attributes \n");
     printf("\t--dropdtd : remove the DOCTYPE of the input docs\n");
+    printf("\t--stream : use the streaming interface to process very large files\n");
+    printf("\t--chkregister : verify the node registration code\n");
+#ifdef LIBXML_SCHEMAS_ENABLED
+    printf("\t--relaxng schema : do RelaxNG validation against the schema\n");
+#endif
     printf("\nLibxml project home page: http://xmlsoft.org/\n");
     printf("To report bugs or get some help check: http://xmlsoft.org/bugs.html\n");
 }
+
+static void registerNode(xmlNodePtr node)
+{
+    node->_private = malloc(sizeof(long));
+    *(long*)node->_private = (long) 0x81726354;
+}
+
+static void deregisterNode(xmlNodePtr node)
+{
+    assert(node->_private != NULL);
+    assert(*(long*)node->_private == (long) 0x81726354);
+    free(node->_private);
+}
+
 int
 main(int argc, char **argv) {
-    int i, count;
+    int i, acount;
     int files = 0;
     int version = 0;
-
+    const char* indent;
+    
     if (argc <= 1) {
 	usage(argv[0]);
 	return(1);
@@ -1105,10 +1303,11 @@ main(int argc, char **argv) {
 
 	if (argv[i][0] != '-')
 	    continue;
-#ifdef LIBXML_DEBUG_ENABLED
 	if ((!strcmp(argv[i], "-debug")) || (!strcmp(argv[i], "--debug")))
 	    debug++;
-	else if ((!strcmp(argv[i], "-shell")) ||
+	else
+#ifdef LIBXML_DEBUG_ENABLED
+	if ((!strcmp(argv[i], "-shell")) ||
 	         (!strcmp(argv[i], "--shell"))) {
 	    shell++;
             noout = 1;
@@ -1256,6 +1455,21 @@ main(int argc, char **argv) {
 	     noblanks++;
 	     format++;
 	     xmlKeepBlanksDefault(0);
+	}
+	else if ((!strcmp(argv[i], "-stream")) ||
+	         (!strcmp(argv[i], "--stream"))) {
+	     stream++;
+	}
+	else if ((!strcmp(argv[i], "-chkregister")) ||
+	         (!strcmp(argv[i], "--chkregister"))) {
+	     chkregister++;
+#ifdef LIBXML_SCHEMAS_ENABLED
+	} else if ((!strcmp(argv[i], "-relaxng")) ||
+	         (!strcmp(argv[i], "--relaxng"))) {
+	    i++;
+	    relaxng = argv[i];
+	    noent++;
+#endif
 	} else {
 	    fprintf(stderr, "Unknown option %s\n", argv[i]);
 	    usage(argv[0]);
@@ -1277,6 +1491,18 @@ main(int argc, char **argv) {
 	}
     }
 #endif
+
+    if (chkregister) {
+	xmlRegisterNodeDefault(registerNode);
+	xmlDeregisterNodeDefault(deregisterNode);
+    }
+    
+    indent = getenv("XMLLINT_INDENT");
+    if(indent != NULL) {
+	xmlTreeIndentString = indent;
+    }
+    
+
     xmlLineNumbersDefault(1);
     if (loaddtd != 0)
 	xmlLoadExtDtdDefaultValue |= XML_DETECT_IDS;
@@ -1296,6 +1522,33 @@ main(int argc, char **argv) {
 	 "<body bgcolor=\"#ffffff\"><h1 align=\"center\">%s output</h1>\n",
 		argv[0]);
     }
+
+#ifdef LIBXML_SCHEMAS_ENABLED
+    if ((relaxng != NULL) && (stream == 0)) {
+	xmlRelaxNGParserCtxtPtr ctxt;
+
+        /* forces loading the DTDs */
+        xmlLoadExtDtdDefaultValue |= 1; 
+	if (timing) {
+	    startTimer();
+	}
+	ctxt = xmlRelaxNGNewParserCtxt(relaxng);
+	xmlRelaxNGSetParserErrors(ctxt,
+		(xmlRelaxNGValidityErrorFunc) fprintf,
+		(xmlRelaxNGValidityWarningFunc) fprintf,
+		stderr);
+	relaxngschemas = xmlRelaxNGParse(ctxt);
+	if (relaxngschemas == NULL) {
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Relax-NG schema %s failed to compile\n", relaxng);
+	    relaxng = NULL;
+	}
+	xmlRelaxNGFreeParserCtxt(ctxt);
+	if (timing) {
+	    endTimer("Compiling the schemas");
+	}
+    }
+#endif
     for (i = 1; i < argc ; i++) {
 	if ((!strcmp(argv[i], "-encode")) ||
 	         (!strcmp(argv[i], "--encode"))) {
@@ -1312,15 +1565,27 @@ main(int argc, char **argv) {
 	    i++;
 	    continue;
         }
+	if ((!strcmp(argv[i], "-relaxng")) ||
+	         (!strcmp(argv[i], "--relaxng"))) {
+	    i++;
+	    continue;
+        }
 	if ((timing) && (repeat))
 	    startTimer();
 	/* Remember file names.  "-" means stdin.  <sven@zen.org> */
 	if ((argv[i][0] != '-') || (strcmp(argv[i], "-") == 0)) {
 	    if (repeat) {
-		for (count = 0;count < 100 * repeat;count++)
+		for (acount = 0;acount < 100 * repeat;acount++)
+		    if (stream != 0)
+			streamFile(argv[i]);
+		    else
+			parseAndPrintFile(argv[i]);
+	    } else {
+		if (stream != 0)
+		    streamFile(argv[i]);
+		else
 		    parseAndPrintFile(argv[i]);
-	    } else
-		parseAndPrintFile(argv[i]);
+	    }
 	    files ++;
 	    if ((timing) && (repeat)) {
 		endTimer("100 iterations");
@@ -1335,6 +1600,11 @@ main(int argc, char **argv) {
     if ((files == 0) && (!generate) && (version == 0)) {
 	usage(argv[0]);
     }
+#ifdef LIBXML_SCHEMAS_ENABLED
+    if (relaxngschemas != NULL)
+	xmlRelaxNGFree(relaxngschemas);
+    xmlRelaxNGCleanupTypes();
+#endif
     xmlCleanupParser();
     xmlMemoryDump();
 
